﻿using ComputersShopContracts.BindingModels;
using ComputersShopContracts.ViewModels;
using ComputersShopDataModels.Enums;
using ComputersShopDataModels.Models;
using System;
using System.Collections.Generic;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Reflection.Metadata;
using System.Text;
using System.Threading.Tasks;

namespace ComputersShopDataBaseImplement.Models
{
	public class Order : IOrderModel
	{
		[Required]
		public int ComputerId { get; set; }
		[Required]
		public int ClientId { get; private set; }
        public int? ImplementerId { get; set; }
        [Required]
		public int Count { get; set; }
		[Required]
		public double Sum { get; set; }
		[Required]
		public OrderStatus Status { get; set; }
		[Required]
		public DateTime DateCreate { get; set; }

		public DateTime? DateImplement { get; set; }
		public virtual Computer Computer { get; set; }
		public virtual Client Client { get; set; }
        public virtual Implementer? Implementer { get; set; }
        public int Id { get; set; }
		public static Order? Create(OrderBindingModel? model)
		{
			if (model == null)
			{
				return null;
			}
			return new Order()
			{
				Id = model.Id,
				ComputerId = model.ComputerId,
				ClientId = model.ClientId,
                ImplementerId = model.ImplementerId,
                Count = model.Count,
				Sum = model.Sum,
				Status = model.Status,
				DateCreate = model.DateCreate,
				DateImplement = model.DateImplement
			};
		}

		public void Update(OrderBindingModel? model)
		{
			if (model == null)
			{
				return;
			}
			Status = model.Status;
			DateImplement = model.DateImplement;
            ImplementerId = model.ImplementerId;
        }

        public OrderViewModel GetViewModel => new()
		{
			Id = Id,
			ComputerId = ComputerId,
			ClientId = ClientId,
			ClientFIO = Client.ClientFIO,
            ImplementerId = ImplementerId,
            ImplementerFIO = Implementer?.ImplementerFIO ?? string.Empty,
            Count = Count,
			Sum = Sum,
			Status = Status,
			DateCreate = DateCreate,
			DateImplement = DateImplement,
			ComputerName = Computer.ComputerName
		};
	}
}
